'use strict';

const WikidotAJAX = require('wikidot-ajax');
const XMLRPC = require('xmlrpc');
const Promise = require('bluebird');
const axios = require('axios');

const DEFAULT_CONCURRENCY = 4;

class WikidotKit {
    /**
     * @param {String} token
     * @param {String} [user]
     * @param {Number} [concurrency]
     */
    constructor({token, user, concurrency = DEFAULT_CONCURRENCY}) {
        if (!token) {
            throw new Error('token is required');
        }

        this.concurrency = concurrency;

        this.xmlrpc = XMLRPC.createSecureClient({
            host: 'www.wikidot.com',
            port: 443,
            path: '/xml-rpc-api.php',
            basic_auth: {
                user: user || `WikidotKit v${WikidotKit.version}`,
                pass: token
            }
        });
    }

    /**
     * XMLRPC API method call
     *
     * @private
     *
     * @param {String} wiki
     * @param {String} method
     * @param {Object} [args]
     *
     * @returns {Promise}
     */
    call({wiki, method, args}) {
        const callArgs = Object.assign({}, args, {site: wiki});

        return new Promise((resolve, reject) => {
            this.xmlrpc.methodCall(method, [callArgs], (error, result) => {
                if (error) {
                    reject(error);
                } else {
                    resolve(result);
                }
            });
        });
    }

    /**
     * @param {String} wiki
     *
     * @returns {Promise}
     */
    fetchPagesList({wiki}) {
        return this.call({wiki, method: 'pages.select'});
    }

    /**
     * Fetches single page metadata and content by wiki and name
     *
     * @param {String} wiki
     * @param {String} name
     *
     * @returns {Promise}
     */
    fetchPage({wiki, name}) {
        return this.call({
            wiki,
            method: 'pages.get_one',
            args: {
                site: wiki,
                page: name
            }
        });
    }

    /**
     * Fetches full list of wiki members
     *
     * @param {String} wikiURL
     *
     * @returns {Promise}
     */
    fetchMembersList({wikiURL}) {
        const query = new WikidotAJAX({baseURL: wikiURL});

        return query({
            moduleName: 'membership/MembersListModule'
        })
            .then(($) => {
                const totalPages = parseInt($('.pager .target:nth-last-child(2)').text(), 10);
                const pages = [];
                for (let i = 1; i <= totalPages; i++) {
                    pages.push(i);
                }
                return pages;
            })

            .map((pageNumber) => {
                return query({
                    moduleName: 'membership/MembersListModule',
                    page: pageNumber
                });
            })

            .map(($) => {
                return Array.from($('.printuser a:last-of-type')).map((elem) => {
                    const jqElem = $(elem);
                    return {
                        username: jqElem.text(),
                        uid: Number(jqElem.attr('onclick').replace(/.*\((.*?)\).*/, '$1'))
                    };
                });
            }, {concurrency: this.concurrency})

            .reduce((allUsers, pageOfUsers) => allUsers.concat(pageOfUsers), []);
    }

    /**
     * Fetches wikidot user profile by uid
     *
     * @param {String} wikiURL
     * @param {Number} uid
     *
     * @returns {Promise}
     */
    fetchUserProfile({wikiURL, uid}) {
        const query = new WikidotAJAX({baseURL: wikiURL});

        return query({
            moduleName: 'users/UserInfoWinModule',
            user_id: uid
        })
            .then(($) => {
                const username = $('h1').text();
                const about = $('.table tr em').text();
                const date = $('.table tr .odate');
                const userSince = new Date($(date[0]).text());
                const memberSince = new Date($(date[1]).text());
                if (username.length) {
                    return {uid, username, about, userSince, memberSince};
                } else {
                    return {uid, deleted: true};
                }
            });
    }

    /**
     * Resolves page id by it's name
     *
     * @param {String} pageURL
     *
     * @returns {Promise}
     */
    async resolvePageID({pageURL}) {
        const PAGE_ID_REGEXP = /WIKIREQUEST\.info\.pageId = (\d+);/;
        const page = await axios({url: pageURL});
        const match = PAGE_ID_REGEXP.exec(page.data);
        return match && parseInt(match[1], 10) || null;
    }

    /**
     * Fetches votes for one page by id
     *
     * @param {String} wikiURL
     * @param {String} pageName
     *
     * @returns {Promise}
     */
    async fetchPageVotes({wikiURL, pageName}) {
        const pageURL = `${wikiURL}/${pageName}`;
        const pageID = await this.resolvePageID({pageURL});
        if (!pageID) {
            return Promise.reject(`Page ${pageName} ID cannot be resolved`);
        }
        const query = new WikidotAJAX({baseURL: wikiURL});

        const $ = await query({
            moduleName: 'pagerate/WhoRatedPageModule',
            pageId: pageID
        });

        const uids = Array.from($('span.printuser a:first-of-type')
            .map((i, a) => {
                const uid = $(a)
                    .attr('onclick')
                    .replace('WIKIDOT.page.listeners.userInfo(', '')
                    .replace('); return false;', '');
                return parseInt(uid, 10);
            }));
        const votes = Array.from($('span:not([class])').map((i, span) => $(span).text().trim()));

        const userVotes = [];

        uids.forEach((uid, i) => {
            userVotes.push({uid, vote: votes[i]});
        });

        return userVotes;
    }

    /**
     * Fetches revisions for one page by id
     *
     * @param {String} wikiURL
     * @param {String} pageName
     *
     * @returns {Promise}
     */
    async fetchPageRevisionsList({wikiURL, pageName}) {
        const pageURL = `${wikiURL}/${pageName}`;
        const pageID = await this.resolvePageID({pageURL});
        if (!pageID) {
            return Promise.reject(`Page ${pageName} ID cannot be resolved`);
        }
        const query = new WikidotAJAX({baseURL: wikiURL});

        const $ = await query({
            moduleName: 'history/PageRevisionListModule',
            page_id: pageID,
            options: {source: true},
            perpage: 3000
        });

        return Array.from($('.page-history tr:not(:first-child)').map((i, row) => {
            const revisionNumber = parseInt($(row).find('td:nth-child(1)').text(), 10);

            const rawUID = $(row)
                    .find('.printuser a:first-child')
                    .attr('onclick');
            const uid = rawUID
                ? parseInt(rawUID.replace('WIKIDOT.page.listeners.userInfo(', '').replace('); return false;', ''), 10)
                : -1;

            const date = $(row).find('td:nth-child(6)').text().replace('\n\t\t', '');
            const description = $(row).find('td:nth-child(7)').text();

            const revisionID = $(row)
                .find('a:first-child')
                .attr('onclick')
                .replace('showVersion(', '')
                .replace(')', '');

            return {number: revisionNumber, id: revisionID, uid, date, description};
        }));
    }

    async fetchPageRevisionContent({wikiURL, revisionID}) {
        const query = new WikidotAJAX({baseURL: wikiURL});

        const $ = await query({
            moduleName: 'history/PageSourceModule',
            revision_id: revisionID
        });

        return unescape($('.page-source').text());
    }

    /**
     * Fetches posts from one thread page
     *
     * @param {String} wikiURL
     * @param {Number} topicID
     * @param {Number} pageNumber
     *
     * @returns {Object}
     */
    fetchThreadPage({wikiURL, topicID, pageNumber}) {
        const query = new WikidotAJAX({baseURL: wikiURL});

        return query({
            moduleName: 'forum/ForumViewThreadPostsModule',
            t: topicID,
            pageNo: pageNumber
        });
    }
}

WikidotKit.AJAX = WikidotAJAX;

module.exports = WikidotKit;
